import glob
import os
import re
import sys

import pytest

current_script = os.path.abspath(__file__)
base_dir = os.path.dirname(current_script)
sys.path.insert(0, os.path.abspath(f"{base_dir}/../tools"))

from parse_fwu import FWUFile

fwu_files = glob.glob("fw_*_*.fwu")

# Skip firmware file content checks for files with the specified MD5 hash
skip_fwu_content_check = [
    "483404ec2238c59756b548d42e162e6c",  # fw_00074_001.fwu / Gateway EtherCAT
    "56b493f1bd5daaf1b9ca5dffca01d36f",  # fw_00075_001.fwu / Gateway Ethernet/IP
    "3a7c20035ac0fd01a380c1557ff8fed0",  # fw_00093_001.fwu / Gateway Modbus TCP
]


@pytest.mark.parametrize("fwu_file", fwu_files)
def test_filename(fwu_file: str) -> None:
    fwu = FWUFile(fwu_file)

    assert re.match(
        r"fw_\d{5}_\d{3}.fwu", fwu_file
    ), "Filename does not match naming convention"

    if fwu.file_md5_hash in skip_fwu_content_check:
        # Skip the check of file content
        return

    _, _module_type, _hardware_revision = fwu_file[:-4].split("_")

    module_type = int(_module_type)
    hardware_revision = int(_hardware_revision)

    assert module_type == fwu.module_type, (
        f"Module type in filename ({module_type}) does not match "
        + f"fwu content ({fwu.module_type})"
    )

    assert hardware_revision == fwu.hardware_revision, (
        f"Hardware revision in filename ({hardware_revision}) does not match "
        + f"fwu content ({fwu.hardware_revision})"
    )


@pytest.mark.parametrize("fwu_file", fwu_files)
def test_fwu_content(fwu_file: str) -> None:
    fwu = FWUFile(fwu_file)

    if fwu.file_md5_hash in skip_fwu_content_check:
        # Skip the check of file content
        return

    assert (
        fwu.version_major >= 1
    ), "Major version of a released firmware must be 1 or greater"
    assert fwu.version_minor >= 0, "Minor version must be 0 or greater"
    assert fwu.hardware_revision >= 1, "Hardware revision must be 1 or greater"

    assert fwu.svn_tag or fwu.git_hash, "Either a SVN tag or a GIT hash must be present"

    assert fwu.fwu_revision in (1, 3), "Invalid FWU revision"
